"""Houses Client Factory"""
from typing import TypeVar, List
from logging import getLogger

from requests import Session

from pyrestsdk.middleware import MiddlewarePipeline
from pyrestsdk.middleware import BaseMiddleware
from pyrestsdk.credential import AbstractBasicCredential
from pyrestsdk.clientfactory._abstract_client_factory import AbstractHTTPClientFactory

Logger = getLogger(__name__)

B = TypeVar("B", bound=BaseMiddleware)
C = TypeVar("C", bound=AbstractBasicCredential)


class HTTPClientFactory(AbstractHTTPClientFactory):
    """HTTP Client Factory"""

    def create_with_custom_middleware(self, middleware: List[B]) -> Session:

        if not middleware:
            raise ValueError("Please provide a list of custom middleware")
        self._register(middleware)
        return self.session

    def create_with_default_middleware(self, credential: C) -> Session:
        return Session()

    def _set_base_url(self, url: str) -> None:
        return

    def _register(self, middleware: List[B]) -> None:
        """
        Helper method that constructs a middleware_pipeline with the specified middleware
        """

        Logger.info("%s._register: method called", type(self))

        if middleware:
            middleware_pipeline = MiddlewarePipeline()
            for ware in middleware:
                middleware_pipeline.add_middleware(ware)

            self.session.mount("https://", middleware_pipeline)



"""Houses Supports Query Options"""

from pyrestsdk.type.model import QueryOptionCollection


class SupportsQueryOptions:
    """Supports Query Options Type"""

    __slots__ = ["_query_options"]

    _query_options: QueryOptionCollection
    
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        
        self._query_options = QueryOptionCollection()

    @property
    def query_options(self) -> QueryOptionCollection:
        """Gets Query Options"""

        return self._query_options



"""Houses Base Entity"""

from __future__ import annotations
from typing import TYPE_CHECKING, Dict, TypeVar, Type
from abc import abstractmethod

if TYPE_CHECKING:
    from pyrestsdk import AbstractServiceClient

S = TypeVar("S", bound="BaseEntity")
A = TypeVar("A", bound="AbstractServiceClient")


class BaseEntity:
    """Base Entity Type"""
    
    __slots__ = ["__client"]
    
    __client: A

    def __init__(self: S, client: A) -> None:
        self.__client = client

    @property
    @abstractmethod
    def as_dict(self) -> Dict:
        """Gets the object as it's dict representation"""

    @property
    @abstractmethod
    def as_json(self) -> str:
        """Gest the object's json representation"""

    @property
    def Client(self: S) -> A:
        """Gets the client"""
        return self.__client

    @classmethod
    @abstractmethod
    def from_json(cls: Type[S], entry: Dict) -> S:
        """Converts Json to class type"""



"""Houses Supports Header Options"""

from pyrestsdk.type.model import HeaderOptionCollection


class SupportsHeaderOptions:
    """Supports Header Options Type"""

    __slots__ = ["_header_options"]

    _header_options: HeaderOptionCollection
    
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        
        self._header_options = HeaderOptionCollection()

    @property
    def header_options(self) -> HeaderOptionCollection:
        """Gets Header Options"""

        return self._header_options
